/*
#---------------------------#
|         PLACAR DE         |
|           TRUCO           |
|       Ítalo  Coelho       |
#---------------------------#
*/

//Bibliotecas----------------
#include <Wire.h>
#include <Adafruit_SSD1306.h>
//---------------------------

//Portas-------------
#define Botao1      7
#define Botao2      5
#define BotaoTruco  3
//-------------------

//Delay---------------
#define atraso    2000
#define debounce   500
//--------------------

//Parâmetros do Display-----
#define SCREEN_WIDTH     128 
#define SCREEN_HEIGHT     32 
#define OLED_RESET         4 
#define OLED_ADDRESS    0x3C
//--------------------------

//Protótipos das Funções----------
void truco();
void placar();
void rodadas();
void nova_partida();
void tela_inicial();
int obter_resposta();
void ambas_com_onze();
void vencedor(int champ);
void mao_de_onze(int equipe_onze);
//--------------------------------

//Variáveis Globais--------------------
bool EstadoAtual_1 = HIGH;
bool EstadoAtual_3 = HIGH;
bool EstadoAtual_Truco = HIGH;
bool EstadoAnterior_1 = HIGH;
bool EstadoAnterior_2 = HIGH;
bool EstadoAnterior_Truco = HIGH;
unsigned short int resposta;
unsigned short int tentos = 0;
unsigned short int pontos[2] = { 0, 0};
//-------------------------------------

//Instancia o display OLED--------------------------------------------
Adafruit_SSD1306 OLED(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);


void setup() 
{
  //Define o padrão das portas onde os botões estão conectados como HIGH
  pinMode(Botao1, INPUT_PULLUP);        
  pinMode(Botao2, INPUT_PULLUP);
  pinMode(BotaoTruco, INPUT_PULLUP);

  //Configura o Display OLED
  OLED.begin(SSD1306_SWITCHCAPVCC, OLED_ADDRESS);
  OLED.setTextColor(WHITE);
  OLED.clearDisplay();
  
  tela_inicial();       //Exibe tela inicial
}


void loop() 
{
  //Jogo Normal
  if(pontos[0] < 11 && pontos[1] < 11)
    rodadas();

  //Equipe 1 Venceu
  if(pontos[0] >= 12)
    vencedor(1);

  //Equipe 2 Venceu
  if(pontos[1] >= 12)
    vencedor(2);
    
  //Equipe 1 com 11 pontos
  if(pontos[0] == 11 && pontos[1] < 11)
    mao_de_onze(1);

  //Equipe 2 com 11 pontos
  if(pontos[1] == 11 && pontos[0] < 11)
    mao_de_onze(2);

  //Ambas as equipes com 11 pontos
  if(pontos[1] == 11 && pontos[0] == 11)
    ambas_com_onze();
}


void placar()
{
  OLED.clearDisplay();                //Limpa a tela
  
  OLED.setTextSize(1);                //Define o tamanho da fonte
  
  OLED.setCursor(2,7);                //Posiciona o cursor
  OLED.print("Equipe 1-->     ");     //Define o texto a ser exibido
  OLED.setCursor(77,7);
  OLED.print(pontos[0]);
  
  OLED.setCursor(2, 16);
  OLED.print("Equipe 2-->     ");
  OLED.setCursor(77, 16);
  OLED.print(pontos[1]);

  OLED.display();                     //Mostra na tela o texto definido
}


int obter_resposta()        
{
  while(true)
  {
    //Lendo o estado dos pinos onde os botões estão conectados
    EstadoAtual_1 = digitalRead(Botao1);
    EstadoAtual_3 = digitalRead(Botao2);
    EstadoAtual_Truco = digitalRead(BotaoTruco);
    
    if(!EstadoAtual_1 && EstadoAnterior_1)            //O botão 1 está apertado
    {
      EstadoAnterior_1 = EstadoAtual_1;
      delay(debounce);                                //Evita que sejam registrados toques duplicados
      return 1;                                       //Função retorna o botão pressionado
      break;                                          //Sai do laço While
    }
    if(!EstadoAtual_3 && EstadoAnterior_2)            //O botão 2 está apertado
    {
      EstadoAnterior_2 = EstadoAtual_3;
      delay(debounce);
      return 2;                                       
      break;
    }
    if(!EstadoAtual_Truco && EstadoAnterior_Truco)    //O botão de truco está apertado
    {
      EstadoAnterior_Truco = EstadoAtual_Truco;
      delay(debounce);
      return 3;
      break;
    }
    
    EstadoAnterior_1 = EstadoAtual_1;
    EstadoAnterior_2 = EstadoAtual_3;
    EstadoAnterior_Truco = EstadoAtual_Truco;
  } 
}


void rodadas()
{
  //Nenhuma equipe com 11 ou vencedora
  while(pontos[0] < 11 && pontos[1] < 11)
  {
    placar();                       //Exibe o placar atual
    resposta = obter_resposta();    //Espera o botão pressionado pelo usuário
    if(resposta == 3)
      truco();                      //A jogada está trucada
    else
      pontos[resposta-1] ++;        //Atualiza o placar 
  }
}


void truco()
{
  tentos += 3;

  //Exibe quanto vale a rodada
  OLED.clearDisplay();
  OLED.setTextSize(2);
 
  switch(tentos)
  {
    case 3:
    OLED.setCursor(3,8);
    OLED.print("  TRUCO!  ");
    break;

    case 6:
    OLED.setCursor(0,8);
    OLED.print("   SEIS!  ");
    break;

    case 9:
    OLED.setCursor(0,8);
    OLED.print("   NOVE!  ");
    break;

    case 12:
    OLED.setCursor(0,8);
    OLED.print("   DOZE!  ");
    break;
  }
  OLED.display();
  delay(1000);
  
  //Desconsidera o botão de truco sendo apertado depois de se pedir 12
  if(tentos > 12)
    tentos = 12;

  //Pergunta quem venceu a rodada trucada
  OLED.clearDisplay();
  OLED.setTextSize(1);
  OLED.setCursor(0,7);
  OLED.print("    Quem Venceu?    ");
  OLED.setCursor(0,16);
  OLED.print("  1)Eq. 1  2)Eq. 2  ");
  OLED.display();
  
  resposta = obter_resposta();
  
  if(resposta == 3)
    truco();    
  else
  {
    pontos[resposta-1] += tentos;
    placar();
    delay(atraso);
  }
  tentos = 0;
}


void mao_de_onze(int equipe_onze)
{
  int outra_equipe;
  if(equipe_onze == 1)
    outra_equipe = 2;
  else
    outra_equipe = 1;
    
  //Pergunta se a equipe com 11 deseja jogar a rodada 
  OLED.clearDisplay();
  OLED.setTextSize(1);
  OLED.setCursor(0,7);
  OLED.print("    Mao de Onze     ");
  OLED.setCursor(0,16);
  OLED.print(" 1)Aceita   2)Corre ");
  OLED.display();
  
  resposta = obter_resposta();
  
  //Rodada foi aceita
  if(resposta == 1)
  {  
    //Pergunta quem venceu a rodada
    OLED.clearDisplay();
    OLED.setTextSize(1);
    OLED.setCursor(0,7);
    OLED.print("    Quem Venceu?    ");
    OLED.setCursor(0,16);
    OLED.print("  1)Eq. 1  2)Eq. 2  ");
    OLED.display();
    
    resposta = obter_resposta();
    
    if(resposta == equipe_onze)
      pontos[equipe_onze-1] += 3;
 
    if(resposta == outra_equipe)
    {
      pontos[outra_equipe-1] += 3;
      placar();
      delay(atraso);
    }
  }

  //Rodada não foi aceita
  else if(resposta == 2)
  {
    pontos[outra_equipe-1]++;   //Equipe em desvantagem recebe um ponto
    placar();
    delay(atraso);
  }
  
}


void ambas_com_onze()
{
  //Pergunta qual equipe venceu a rodada final
  OLED.clearDisplay();
  OLED.setTextSize(1);
  OLED.setCursor(0,7);
  OLED.print("   Ambas com Onze   ");
  OLED.display();
  delay(atraso);
  
  OLED.clearDisplay();
  OLED.setTextSize(1);
  OLED.setCursor(0,7);
  OLED.print("    Quem Venceu?    ");
  OLED.setCursor(0,16);
  OLED.print("  1)Eq. 1  2)Eq. 2  ");
  OLED.display();

  resposta = obter_resposta();
  
  if(resposta == 1)
    pontos[0]++;

  if(resposta == 2)
    pontos[1]++;
}


void vencedor(int champ)
{
  //Exibe a equipe vencedora
  for(int i = 0; i < 4; i++)
  {
    OLED.clearDisplay();
    OLED.setTextSize(1);
    OLED.setCursor(0,7);
    OLED.print("    O Jogo Acabou!   ");
    OLED.setCursor(0,16);
    OLED.print("                     ");
    OLED.display();
    delay(atraso);
    OLED.setCursor(0,16);
    OLED.print("   Equipe   Venceu!  ");
    OLED.setCursor(60,16);
    OLED.print(champ);
    OLED.display();
    delay(atraso);
  }
  OLED.setCursor(0,7);
  OLED.println("        Italo        ");
  OLED.setCursor(0,15);
  OLED.println("        Coelho       ");
  OLED.display();
}


void tela_inicial()     //Animação Incial
{
  OLED.setTextSize(1);
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println("---------------------");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println(" ------------------- ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println("  -----------------  ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println("   ---------------   ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println("    -------------    ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println("    -------------    ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println("      ---------      ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println("      ---------      ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println("        -----        ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println("         ---         ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,15);
  OLED.println("          -          ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
  OLED.setCursor(0,7);
  OLED.println("   Placar de Truco   ");
  OLED.setCursor(0,15);
  OLED.println("                     ");
  OLED.display();
  delay(50);
  OLED.clearDisplay();
}
